"""Tests for version information."""

import pytest

from ai_org.__version__ import __version__


@pytest.mark.unit
class TestVersion:
    def test_version_is_string(self):
        assert isinstance(__version__, str)

    def test_version_not_empty(self):
        assert len(__version__) > 0
